@@ -11,6 +11,11 @@ from mx.Log import *
 #import TraceCalls; TraceCalls.install()
 #log.setup(ignore_level=SYSTEM_LOG_EVERYTHING)
 
+### Globals
+
+# Write debug info
+_debug = 0
+
 ### Constants
 
 # Special keys
@@ -72,15 +77,33 @@ class BeeBaseDict:
         to write the cache data to disk storage.
         
     """
-    name = 'NoName'                             # Name of the dict
-    index = None                                # Index object
-    storage = None                              # Storage object
-    closed = 0                                  # Are we closed ?
-    cache = None                                # Cache dict
-    readonly = 0                                # Are we read-only ?
-    autocommit = 0                              # Autocommit ? (this will
-                                                # perform a commit whenever
-                                                # the cache gets to full)
+    # Name of the dictionary
+    name = 'NoName'
+
+    # Name of the storage file; set in .__init__()
+    storage_name = name + '.dat'
+
+    # Name of the index file; set in .__init__()
+    index_name = name + '.idx'
+
+    # Bee*Index object
+    index = None
+
+    # Bee*Storage object
+    storage = None
+
+    # Is the dictionary closed ?
+    closed = 0
+
+    # Cache dictionary
+    cache = None
+
+    # Is the dictionary read-only ?
+    readonly = 0
+
+    # Run in auto-commit mode ? This will perform a commit whenever
+    # the cache gets to full.
+    autocommit = 0                              
 
     # Max. cache size in number of items to store in the in-memory
     # transaction cache
@@ -132,6 +155,8 @@ class BeeBaseDict:
         """
         # Init instance vars
         self.name = name
+        self.storage_name = name + '.dat'
+        self.index_name = name + '.idx'
         self.cache = {}
         if maxcachesize is not None:
             self.maxcachesize = maxcachesize
@@ -140,14 +165,14 @@ class BeeBaseDict:
 
         # Open storage
         self.storage = BeeStorage.BeeKeyValueStorage(
-                                name + '.dat',
-                                lock= not readonly,
-                                cache=0,
-                                min_recordsize=min_recordsize,
-                                readonly=readonly,
-                                recover=recover)
+            self.storage_name,
+            lock= not readonly,
+            cache=0,
+            min_recordsize=min_recordsize,
+            readonly=readonly,
+            recover=recover)
         
-        # Determine the filemode
+        # Determine the filemode for the index
         if readonly:
             # Readonly mode
             filemode = 1
@@ -156,8 +181,8 @@ class BeeBaseDict:
                 # Create a new file (overwriting a possibly existing one)
                 filemode = 2
             else:
-                # Open a file, create is non-existent
-                filemode = 3
+                # Open an existing file, recreate if non-existing
+                filemode = 0
                 
         # Open the index
         try:
@@ -175,7 +200,7 @@ class BeeBaseDict:
                     sectorsize = 1024
                 else:
                     raise IndexError, 'keysize %i is too large' % keysize
-                self.index = index(name + '.idx',
+                self.index = index(self.index_name,
                                    keysize,
                                    dupkeys=0,
                                    filemode=filemode,
@@ -183,14 +208,14 @@ class BeeBaseDict:
                 
             elif index is BeeIndex.BeeIntegerIndex:
                 # keysize is sizeof(long)
-                self.index = index(name + '.idx',
+                self.index = index(self.index_name,
                                    dupkeys=1,
                                    filemode=filemode,
                                    sectorsize=256)
 
             elif index is BeeIndex.BeeFloatIndex:
                 # keysize is sizeof(double)
-                self.index = index(name + '.idx',
+                self.index = index(self.index_name,
                                    dupkeys=1,
                                    filemode=filemode,
                                    sectorsize=256)
@@ -236,8 +261,8 @@ class BeeBaseDict:
 
         """
         self.close()
-        os.remove(self.name + '.dat')
-        os.remove(self.name + '.idx')
+        os.remove(self.storage_name)
+        os.remove(self.index_name)
 
     def __len__(self):
 
@@ -992,9 +1017,6 @@ class BeeDict(BeeBaseDict):
 
         """ Commit all changes and start a new transaction.
         """
-        if __debug__:
-            log(SYSTEM_DEBUG,'Committing all changes for "%s"...',self.name)
-
         # Write all changed entries in the cache to disk
         index = self.index
         index_cursor = index.cursor
@@ -1006,12 +1028,15 @@ class BeeDict(BeeBaseDict):
         storage_delete = storage.delete
         readonly = self.readonly
         NotFound = None
+        debug = _debug
+        if debug:
+            log(SYSTEM_DEBUG,'Committing all changes for "%s"...',self.name)
 
         for key,(state,value) in self.cache.items():
 
             if state == MODIFIED:
-                if __debug__:
-                    log.object(SYSTEM_DEBUG,' writing key "%s":' % repr(key), value)
+                if debug:
+                    log.object(SYSTEM_DEBUG,' writing key %r:' % repr(key), value)
                 if readonly:
                     raise Error,'dict is read-only'
                 hashvalue = hash(key)
@@ -1040,8 +1065,8 @@ class BeeDict(BeeBaseDict):
                     index[hashvalue] = address
 
             elif state == DELETED:
-                if __debug__:
-                    log(SYSTEM_DEBUG,' deleting key "%s"',key)
+                if debug:
+                    log(SYSTEM_DEBUG,' deleting key %r', repr(key))
                 if readonly:
                     raise Error,'dict is read-only'
                 hashvalue = hash(key)
@@ -1310,9 +1335,6 @@ class BeeStringDict(BeeBaseDict):
 
         """ Commit all changes and start a new transaction.
         """
-        if __debug__:
-            log(SYSTEM_DEBUG,'Committing all changes for "%s"...',self.name)
-
         # Write all changed entries in the cache to disk
         index = self.index
         index_get = index.get
@@ -1321,12 +1343,15 @@ class BeeStringDict(BeeBaseDict):
         storage_delete = storage.delete
         readonly = self.readonly
         NotFound = None
+        debug = _debug
+        if debug:
+            log(SYSTEM_DEBUG,'Committing all changes for "%s"...',self.name)
 
         for key,(state,value) in self.cache.items():
 
             if state == MODIFIED:
-                if __debug__:
-                    log.object(SYSTEM_DEBUG,' writing key "%s":' % repr(key), value)
+                if debug:
+                    log.object(SYSTEM_DEBUG,' writing key %s:' % repr(key), value)
                 address = index_get(key,NotFound)
                 if address is not NotFound:
                     # Update storage
@@ -1342,8 +1367,8 @@ class BeeStringDict(BeeBaseDict):
                     index[key] = address
 
             elif state == DELETED:
-                if __debug__:
-                    log(SYSTEM_DEBUG,' deleting key "%s"',key)
+                if debug:
+                    log(SYSTEM_DEBUG,' deleting key %s', repr(key))
                 if readonly:
                     raise Error,'dict is read-only'
                 address = index_get(key,NotFound)
