@@ -526,49 +526,55 @@ class BeeStorage:
             self.mark(HOT)
 
         # Record is to be updated
+        recordsize = None
+        datasize = None
         if position < EOF:
-            recordsize,rtype,datasize = self.read_header(position)
+            recordsize,rt,datasize = self.read_header(position)
             if datasize < datalen:
-                # Drop old record
-                if rtype == VALID:
+                # Mark valid record as old
+                if rt == VALID:
                     if _debug:
                         log(SYSTEM_INFO,
                             'Could not update %i record in place: '
-                            'old size = %i, datalen = %i, data = %s',
+                            'old size = %i, datalen = %i, data = %r',
                             position,datasize,datalen,data)
                     self.free(position)
                 # revert to appending...
                 position = EOF
-            else:
-                # Write new data
-                file.write(data)
-                if _debug:
-                    log(SYSTEM_INFO,'Data written to position %i: %s',
-                        position,data)
-                return position
+                recordsize = None
+                datasize = None
+                
+        elif position > EOF:
+            raise ValueError('cannot write to position outside file: '
+                             'position=%r, EOF=%r' %
+                             (position, EOF))
 
-        # Record is to be appended
-        recordsize = datalen + 6
-        if recordsize < self.min_recordsize:
-            recordsize = self.min_recordsize
-        if recordsize % BLOCKSIZE != 0:
-            recordsize = (recordsize / BLOCKSIZE + 1) * BLOCKSIZE
+        # Calculate recordsize, if not available
+        if recordsize is None:
+            recordsize = datalen + 6
+            if recordsize < self.min_recordsize:
+                recordsize = self.min_recordsize
+            if recordsize % BLOCKSIZE != 0:
+                recordsize = (recordsize / BLOCKSIZE + 1) * BLOCKSIZE
+            datasize = recordsize - 6
 
         # Write the header + data + padding
         file.seek(position)
-        file.write(join((ID,pack('<l',recordsize),rtype,        # Header
-                         data,                                  # Data
-                         '\0' * (recordsize - 6 - datalen)      # Padding
+        file.write(join((ID,pack('<l', recordsize), rtype,     # Header
+                         data,                                 # Data
+                         '\0' * (datasize - datalen)           # Padding
                          ),''))
         if self.caching:
-            self.header_cache.put(position,(recordsize,rtype,recordsize-6))
+            self.header_cache.put(position, (recordsize, rtype, datasize))
 
         # Update EOF
-        self.EOF = file.tell()
+        if position >= EOF:
+            self.EOF = file.tell()
+            if _debug:
+                log(SYSTEM_DEBUG,'New EOF = %i',self.EOF)
         if _debug:
-            log(SYSTEM_DEBUG,'New EOF = %i',self.EOF)
-            log(SYSTEM_DEBUG,'Data written to position %i: %s',
-                position,data)
+            log(SYSTEM_DEBUG,'Data written to position %i: %r',
+                position, data)
 
         return position
 
@@ -585,7 +591,9 @@ class BeeStorage:
         file = self.file
         recordsize,rt,datasize = read_header(position)
         if rtype != rt:
-            raise Error,'record has wrong type'
+            raise Error(
+                'record has wrong type, expected %r, found %r' %
+                (rtype, rt))
         
         # Read the record
         return file.read(datasize)
